@@ -24,14 +24,14 @@ import numpy as np
 import pandas as pd
 
 from concurrent import futures
+from diskcache import Cache
 from requests_futures.sessions import FuturesSession
 from scipy import ndimage
-from tqdm.auto import tqdm
 
 from .. import spine
 from .. import xform
 
-from .utils import parse_volume, FLYWIRE_DATASETS
+from .utils import parse_volume, FLYWIRE_DATASETS, get_chunkedgraph_secret
 
 try:
     import skeletor as sk
@@ -43,7 +43,7 @@ except BaseException:
 __all__ = ['fetch_edit_history', 'fetch_leaderboard', 'locs_to_segments',
            'locs_to_supervoxels', 'skid_to_id', 'update_ids',
            'roots_to_supervoxels', 'supervoxels_to_roots',
-           'neuron_to_segments']
+           'neuron_to_segments', 'is_latest_root']
 
 
 def fetch_leaderboard(days=7, by_day=False, progress=True, max_threads=4):
@@ -96,10 +96,10 @@ def fetch_leaderboard(days=7, by_day=False, progress=True, max_threads=4):
         futures.append(future_session.get(url, params=None))
 
     # Get the responses
-    resp = [f.result() for f in tqdm(futures,
-                                     desc='Fetching',
-                                     disable=not progress or len(futures) == 1,
-                                     leave=False)]
+    resp = [f.result() for f in navis.config.tqdm(futures,
+                                                  desc='Fetching',
+                                                  disable=not progress or len(futures) == 1,
+                                                  leave=False)]
 
     df = None
     for i, r in enumerate(resp):
@@ -164,7 +164,8 @@ def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
 
     session = requests.Session()
     future_session = FuturesSession(session=session, max_workers=max_threads)
-    token = cv.secrets.chunkedgraph_credentials['token']
+
+    token = get_chunkedgraph_secret()
     session.headers['Authorization'] = f"Bearer {token}"
 
     futures = []
@@ -175,10 +176,10 @@ def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
         futures.append(f)
 
     # Get the responses
-    resp = [f.result() for f in tqdm(futures,
-                                     desc='Fetching',
-                                     disable=not progress or len(futures) == 1,
-                                     leave=False)]
+    resp = [f.result() for f in navis.config.tqdm(futures,
+                                                  desc='Fetching',
+                                                  disable=not progress or len(futures) == 1,
+                                                  leave=False)]
 
     df = []
     for r, i in zip(resp, x):
@@ -200,13 +201,16 @@ def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
     return df
 
 
-def roots_to_supervoxels(x, dataset='production', progress=True):
+def roots_to_supervoxels(x, use_cache=True, dataset='production', progress=True):
     """Get supervoxels making up given neurons.
 
     Parameters
     ----------
     x :             int | list of int
                     Segmentation (root) ID(s).
+    use_cache :     bool
+                    Whether to use disk cache to avoid repeated queries for the
+                    same root. Cache is stored in `~/.fafbseg/`.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -237,24 +241,57 @@ def roots_to_supervoxels(x, dataset='production', progress=True):
     # Get the volume
     vol = parse_volume(dataset)
 
-    # Get the supervoxels
-    svoxels = {i: vol.get_leaves(i,
-                                 bbox=vol.meta.bounds(0),
-                                 mip=0) for i in tqdm(x,
-                                                      desc='Querying',
-                                                      disable=not progress,
-                                                      leave=False)}
+    svoxels = {}
+    # See what ewe can get from cache
+    if use_cache:
+        # Cache for root -> supervoxels
+        # Grows to max 1Gb by default and persists across sessions
+        with Cache(directory='~/.fafbseg/svoxel_cache/') as sv_cache:
+            # See if we have any of these roots cached
+            with sv_cache.transact():
+                is_cached = np.isin(x, sv_cache)
+
+            # Add supervoxels from cache if we have any
+            if np.any(is_cached):
+                # Get values from cache
+                with sv_cache.transact():
+                    svoxels.update({i: sv_cache[i] for i in x[is_cached]})
+
+    # Get the supervoxels for the roots that are still missing
+    # We need to convert keys to integer array because otherwise there is a
+    # mismatch in types (int vs np.int?) which causes all root IDs to be in miss
+    # -> I think that's because of the way disk cache works
+    miss = x[~np.isin(x, np.array(list(svoxels.keys())).astype(int))]
+    svoxels.update({i: vol.get_leaves(i,
+                                      bbox=vol.meta.bounds(0),
+                                      mip=0) for i in navis.config.tqdm(miss,
+                                                           desc='Querying',
+                                                           disable=not progress,
+                                                           leave=False)})
+
+    # Update cache
+    if use_cache:
+        with sv_cache.transact():
+            for i in miss:
+                sv_cache[i] = svoxels[i]
 
     return svoxels
 
 
-def supervoxels_to_roots(x, dataset='production'):
+def supervoxels_to_roots(x, use_cache=False, dataset='production'):
     """Get root(s) for given supervoxel(s).
 
     Parameters
     ----------
     x :             int | list of int
                     Supervoxel ID(s) to find the root(s) for.
+    use_cache :     bool
+                    Whether to use disk cache to avoid repeated queries for the
+                    same supervoxel. The implementation for this is optimized
+                    for a small memory footprint - so it's slow! At this point
+                    only use it if you want to minimize your impact on the
+                    flywire backends and you have time on your hands. The cache
+                    is stored in `~/.fafbseg/`.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -275,14 +312,51 @@ def supervoxels_to_roots(x, dataset='production'):
 
     """
     # Make sure we are working with an array of integers
-    x = navis.utils.make_iterable(x).astype(int, copy=False)
+    x = navis.utils.make_iterable(x).astype(np.int64, copy=False)
 
     # Parse the volume
     vol = parse_volume(dataset)
 
-    # get_roots() doesn't like to be asked for zeros - causes server error
+    # Prepare results array
     roots = np.zeros(x.shape, dtype=np.int64)
-    roots[x != 0] = vol.get_roots(x[x != 0])
+
+    # We can't query supervoxel ID 0
+    not_zero = x != 0
+
+    if use_cache:
+        # Cache for supervoxel -> root map
+        # Grows to max 1Gb by default and persists across sessions
+        with Cache(directory='~/.fafbseg/roots_cache/') as roots_cache:
+            # See if we have any of these supervoxels cached
+            with roots_cache.transact():
+                is_cached = np.isin(x, roots_cache)
+
+            # For cached roots, check if they are latest
+            if np.any(is_cached):
+                # Get values from cache
+                with roots_cache.transact():
+                    cached = np.array([roots_cache[i] for i in x[is_cached]])
+                # Check if cached roots are latest
+                is_latest = is_latest_root(cached, dataset=dataset)
+                # Set roots that are still up-to-date
+                is_cached[is_cached] = is_latest
+                roots[is_cached] = cached[is_latest]
+
+            # To fetch are those supervoxels that are != 0 and are not cached
+            to_fetch = ~is_cached & not_zero
+
+            # Fill in the blanks
+            if np.any(to_fetch):
+                roots[to_fetch] = vol.get_roots(x[to_fetch])
+
+                # Update cache
+                with roots_cache.transact():
+                    for k, v in zip(x[to_fetch], roots[to_fetch]):
+                        roots_cache[k] = v
+
+    else:
+        # get_roots() doesn't like to be asked for zeros - causes server error
+        roots[not_zero] = vol.get_roots(x[not_zero])
 
     return roots
 
@@ -514,17 +588,64 @@ def skid_to_id(x,
                         columns=['old_id', 'new_id', 'confidence', 'changed'])
 
 
+def is_latest_root(id, dataset='production', **kwargs):
+    """Check if root is the current one.
+
+    Parameters
+    ----------
+    id :            int | list-like
+                    Single ID or list of flywire (root) IDs.
+    dataset :       str | CloudVolume
+                    Against which flywire dataset to query:
+                      - "production" (current production dataset, fly_v31)
+                      - "sandbox" (i.e. fly_v26)
+
+    Returns
+    -------
+    numpy array
+                    Array of booleans
+
+    See Also
+    --------
+    :func:`~fafbseg.flywire.update_ids`
+                    If you want the new ID.
+
+    Examples
+    --------
+    >>> from fafbseg import flywire
+    >>> flywire.is_latest_root(720575940621039145)
+    array([True])
+
+    """
+    dataset = FLYWIRE_DATASETS.get(dataset, dataset)
+
+    id = navis.utils.make_iterable(id).astype(str)
+
+    session = requests.Session()
+    token = get_chunkedgraph_secret()
+    session.headers['Authorization'] = f"Bearer {token}"
+
+    url = f'https://prodv1.flywire-daf.com/segmentation/api/v1/table/{dataset}/is_latest_roots?int64_as_str=1'
+    post = {'node_ids': id.tolist()}
+    r = session.post(url, json=post)
+
+    r.raise_for_status()
+
+    return np.array(r.json()['is_latest'])
+
+
 def update_ids(id,
                sample=0.1,
                dataset='production',
                progress=True, **kwargs):
-    """Retrieve the most recent version of given flywire neuron(s).
+    """Retrieve the most recent version of given FlyWire (root) ID(s).
 
     This function works by:
-        1. Fetching all supervoxels for a given to-be-updated ID
-        2. Picking a random sample of ``sample`` of these supervoxels
-        3. Fetching the most recent root IDs for the sample supervoxels
-        4. Returning the root ID that was hit the most.
+        1. Checking if ID is outdated (see fafbseg.flywire.is_latest_root)
+        2. Fetching all supervoxels for outdated IDs
+        3. Picking a random sample of ``sample`` of these supervoxels
+        4. Fetching the most recent root IDs for the sample supervoxels
+        5. Returning the root ID that was hit the most.
 
     Parameters
     ----------
@@ -549,6 +670,11 @@ def update_ids(id,
                     0
                     1
 
+    See Also
+    --------
+    :func:`~fafbseg.flywire.is_latest_root`
+                    If all you want is to know whether a (root) ID is up-to-date.
+
     Examples
     --------
     >>> from fafbseg import flywire
@@ -559,51 +685,66 @@ def update_ids(id,
     """
     assert sample > 0, '`sample` must be > 0'
 
+    # See if we already check if this was the latest root
+    is_latest = kwargs.pop('is_latest', None)
+
     vol = parse_volume(dataset, **kwargs)
 
     if isinstance(id, (list, set, np.ndarray)):
+        is_latest = is_latest_root(id, dataset=dataset)
         res = [update_ids(x,
-                          vol=vol,
-                          sample=sample) for x in tqdm(id,
-                                                       desc='Updating',
-                                                       leave=False,
-                                                       disable=not progress or len(id) == 1)]
-        return pd.concat(res, axis=0, sort=False)
+                          dataset=vol,
+                          is_latest=il,
+                          sample=sample) for x, il in navis.config.tqdm(zip(id, is_latest),
+                                                           desc='Updating',
+                                                           leave=False,
+                                                           total=len(id),
+                                                           disable=not progress or len(id) == 1)]
+        return pd.concat(res, axis=0, sort=False, ignore_index=True)
 
-    # Get supervoxel ids - we need to use mip=0 because otherwise small neurons
-    # might not have any (visible) supervoxels
-    svoxels = vol.get_leaves(id, bbox=vol.meta.bounds(0), mip=0)
+    # Check if outdated
+    if isinstance(is_latest, type(None)):
+        is_latest = is_latest_root(id, dataset=dataset)[0]
 
-    # Shuffle voxels
-    np.random.shuffle(svoxels)
+    if not is_latest:
+        # Get supervoxel ids - we need to use mip=0 because otherwise small neurons
+        # might not have any (visible) supervoxels
+        svoxels = vol.get_leaves(id, bbox=vol.meta.bounds(0), mip=0)
 
-    # Generate sample
-    if sample >= 1:
-        smpl = svoxels[: sample]
-    else:
-        smpl = svoxels[: int(len(svoxels) * sample)]
+        # Shuffle voxels
+        np.random.shuffle(svoxels)
 
-    # Fetch up-to-date root IDs for the sampled supervoxels
-    roots = supervoxels_to_roots(smpl, dataset=vol)
+        # Generate sample
+        if sample >= 1:
+            smpl = svoxels[: sample]
+        else:
+            smpl = svoxels[: int(len(svoxels) * sample)]
 
-    # Find unique Ids and count them
-    unique, counts = np.unique(roots, return_counts=True)
+        # Fetch up-to-date root IDs for the sampled supervoxels
+        roots = supervoxels_to_roots(smpl, dataset=vol)
 
-    # Get sorted indices
-    sort_ix = np.argsort(counts)
+        # Find unique Ids and count them
+        unique, counts = np.unique(roots, return_counts=True)
 
-    # New Id is the most frequent ID
-    new_id = unique[sort_ix[-1]]
+        # Get sorted indices
+        sort_ix = np.argsort(counts)
 
-    # Confidence is the difference between the top and the 2nd most frequent ID
-    if len(unique) > 1:
-        conf = round((counts[sort_ix[-1]] - counts[sort_ix[-2]]) / sum(counts),
-                     2)
+        # New Id is the most frequent ID
+        new_id = unique[sort_ix[-1]]
+
+        # Confidence is the difference between the top and the 2nd most frequent ID
+        if len(unique) > 1:
+            conf = round((counts[sort_ix[-1]] - counts[sort_ix[-2]]) / sum(counts),
+                         2)
+        else:
+            conf = 1
     else:
+        new_id = id
         conf = 1
 
     return pd.DataFrame([[id, new_id, conf, id != new_id]],
-                        columns=['old_id', 'new_id', 'confidence', 'changed'])
+                        columns=['old_id', 'new_id', 'confidence', 'changed']
+                        ).astype({'old_id': int, 'new_id': int})
 
 
 def snap_to_id(locs, id, snap_zero=False, dataset='production',
@@ -668,7 +809,7 @@ def snap_to_id(locs, id, snap_zero=False, dataset='production',
         to_fix = to_fix & not_zero
 
     # Use parallel processes to go over the to-fix nodes
-    with tqdm(desc='Snapping', total=to_fix.sum(), leave=False) as pbar:
+    with navis.config.tqdm(desc='Snapping', total=to_fix.sum(), leave=False) as pbar:
         with futures.ProcessPoolExecutor(max_workers=max_workers) as ex:
             loc_futures = [ex.submit(_process_cutout,
                                      id=id,
